/* json.js - Parse JSON With Kouprey
 * Copyright (C) 2009 Rob King <jking@deadpixi.com> 
 *
 * This file is part of Kouprey.
 *
 * Kouprey is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kouprey is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Kouprey.  If not, see <http://www.gnu.org/licenses/>.
 */

/* This file builds on the parsing library to generate a full Abstract
 * Syntax Tree (AST) from the input. This AST could then be transformed
 * by the rest of the program (in the case of JSON, presumably into
 * deserialized objects. */

/* Load the Kouprey library. */
load(["../../com/deadpixi/kouprey/kouprey.js"]);

/* Load the JSON serialization code. Note that we just use this to spit out
 * the AST; we don't use this for any parsing. */
load(["../lib/json.js"]);

/* Load the beautifier code. */
load(["../lib/beautify.js"]);

/* Declare a dictionary for our rules. */
var Rules = {};

/* Write the grammar. */
with (com.deadpixi.kouprey) {
    with (Rules) {
        Rules.Obj   = Forward("Obj");
        Rules.Arr   = Forward("Arr");

        Rules.Space = /^[\x20\x09\x0d\x0a]*/;
        
        Rules.Num   = $(/^\-?(0|[1-9][0-9]*)(\.[0-9]+)?([Ee][+\-]?[0-9]+)?/,
                        "number");

        Rules.Chr   = Or($(/^\\(["\\/bfnrt]|u[0-9A-Fa-f]{4})/, "char"),
                         $(/^[^"\\]*/, "char"));

        Rules.Str   = ['"', $(Any(Chr), "string"), '"'];

        Rules.Val   = Or(Str, Num, $("true", "bool"), $("false", "bool"), 
                         $("null"), Obj, Arr);

        Rules.Arr   = $(["[", Space,
                         Optional([Any([Space, Val, Space, ","]),
                                   Space, Val]),
                         Space, "]"], "array");

        Rules.Pair  = $([Space, Str, Space, ":", Space, Val, Space],
                        "pair");

        Rules.Obj   = $(["{",Optional([Any([Space, Pair, Space, ","]),
                                    Pair]), "}"], "object");

        Rules.JSON  = $(Or(Obj, Arr), "json");
    }
}

/* Parse the input. */
var input = File.input.readAll().join("\n");

/* Create the parser. */
var parser = new com.deadpixi.kouprey.Parser(Rules, "JSON");

/* Parse the input. */
var value = parser.parse(input);

/* Print the resulting value. */
if (value === null) {
    print("ERROR at offset " + parser.highestOffset);
} else {
    print(js_beautify(JSON.stringify(value)));
}
