/* json-grammar.js - Parse JSON With Kouprey
 * Copyright (C) 2009 Rob King <jking@deadpixi.com> 
 *
 * This file is part of Kouprey.
 *
 * Kouprey is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kouprey is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Kouprey.  If not, see <http://www.gnu.org/licenses/>.
 */

/* This file declares the full JSON grammar using Kouprey. Notice how similar
 * the grammar declaration is to EBNF...*/

/* Declare a dictionary for our rules. */
var Rules = {};

/* Write the grammar. */
with (com.deadpixi.kouprey) {
    with (Rules) {
        Rules.Obj   = Forward("Obj");
        Rules.Arr   = Forward("Arr");

        Rules.Space = /^[\x20\x09\x0d\x0a]*/;
        
        Rules.Num   = /^\-?(0|[1-9][0-9]*)(\.[0-9]+)?([Ee][+\-]?[0-9]+)?/;

        Rules.Str   = ['"',
                       Any(Or(/^\\(["\\/bfnrt]|u[0-9A-Fa-f]{4})/,
                              /^[^"\\]*/)),
                       '"'];

        Rules.Val   = Or(Str, Num, "true", "false", "null", Obj, Arr);

        Rules.Arr   = ["[", Space,
                       Optional([Any([Space, Val, Space, ","]),
                                 Space, Val]),
                       Space, "]"];

        Rules.Obj   = ["{", 
                        Optional([Any([Space,
                                       Str, Space, ":",
                                       Space, Val, Space, ","]),
                                  Space,
                                  Str, Space, ":", Space, Val]),
                         Space, "}"];

        Rules.JSON  = Or(Obj, Arr);
    }
}

